SystemOrganization addCategory: #'Iliad-Swazoo'!

SwazooSite subclass: #ILSite
	instanceVariableNames: 'codec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Iliad-Swazoo'!

!ILSite class methodsFor: 'defaults' stamp: 'np 12/3/2008 15:23'!
default
	self ensureRegistered.
	^self named: self defaultName! !

!ILSite class methodsFor: 'defaults' stamp: 'np 12/3/2008 15:19'!
defaultHost
	^'*'! !

!ILSite class methodsFor: 'defaults' stamp: 'np 12/3/2008 15:19'!
defaultIp
	^'*'! !

!ILSite class methodsFor: 'defaults' stamp: 'np 12/3/2008 15:19'!
defaultName
	^'IliadSite'! !

!ILSite class methodsFor: 'defaults' stamp: 'np 12/3/2008 15:25'!
defaultPort
	^8000! !

!ILSite class methodsFor: 'defaults' stamp: 'np 12/3/2008 15:24'!
ensureRegistered
	(self named: self defaultName) ifNil: [
		self newDefaultOn: self defaultPort]! !

!ILSite class methodsFor: 'instance creation' stamp: 'np 10/25/2008 15:12'!
new
	^self basicNew! !

!ILSite class methodsFor: 'instance creation' stamp: 'np 12/3/2008 15:21'!
newDefaultOn: aPort
	^(self newNamed: self defaultName)
		host: self defaultHost
		ip: self defaultIp
		port: aPort! !

!ILSite methodsFor: 'serving' stamp: 'np 4/5/2010 18:23'!
answerTo: aRequest
	[ILDispatcher current 
		dispatch: (self convertRequest: aRequest)]
			on: ILResponseNotification
			do: [:n | ^self convertResponse: n response]! !

!ILSite methodsFor: 'accessing' stamp: 'np 4/5/2010 17:55'!
codec
	^codec ifNil: [codec := self defaultCodec]! !

!ILSite methodsFor: 'accessing' stamp: 'np 4/5/2010 17:55'!
codec: aCodec
	codec := aCodec! !

!ILSite methodsFor: 'private-serving' stamp: 'np 12/4/2008 17:14'!
convertHeader: aSwazooRequest
	| swazooFields fields |
	swazooFields := aSwazooRequest headers fields.
	fields := Dictionary new: swazooFields size.
	swazooFields keysAndValuesDo: [:key :field |
		| value |
		value := String streamContents: [:stream |
			field valuesAsStringOn: stream ].
		fields at: key asLowercase put: value].
	^fields! !

!ILSite methodsFor: 'private-serving' stamp: 'np 5/1/2009 14:35'!
convertPostDataFrom: aValue
	aValue filename isEmptyOrNil
		ifTrue: [^self decode: aValue value]
		ifFalse: [
				^ILFileProxy new
					filename: aValue filename;
					contentType: aValue contentType;
					contents: aValue value;
					yourself]! !

!ILSite methodsFor: 'private-serving' stamp: 'np 4/13/2009 21:32'!
convertRequest: aSwazooRequest
	^ILRequest new
		method: aSwazooRequest methodName;
		url: (ILUrl absolute: aSwazooRequest uriString);
		headers: (self convertHeader: aSwazooRequest);
		fields: (self parseFields: aSwazooRequest);
		cookies: (self parseCookies: aSwazooRequest);
		nativeRequest: aSwazooRequest;
		yourself! !

!ILSite methodsFor: 'private-serving' stamp: 'np 4/5/2010 17:38'!
convertResponse: aResponse
	| swazooResponse |
	swazooResponse := HTTPResponse new.
			
	aResponse headers do: [:each |
		| field fieldClass |
		fieldClass := HeaderField classForFieldName: each key.
		field := fieldClass newForFieldName: each key withValueFrom: each value.
		swazooResponse headers addField: field].
	aResponse cookies do: [:each | 
		swazooResponse cookie: (self cookieHeaderFor: each)].
	swazooResponse code: aResponse status.

	swazooResponse entity: ((self isBinaryResponse: aResponse)
		ifTrue: [aResponse contents]
		ifFalse: [self encode: aResponse contents]).
	swazooResponse contentType: aResponse contentType greaseString.

	aResponse release.
	^swazooResponse! !

!ILSite methodsFor: 'private-serving' stamp: 'np 4/2/2009 21:23'!
cookieHeaderFor: aCookie
	^String streamContents: [:stream |
		stream
			nextPutAll: aCookie key;
			nextPutAll: '=';
			nextPutAll: (aCookie value ifNil: ['']).
		aCookie path isNil ifFalse: [
			stream
				nextPutAll: '; path=';
				nextPutAll: aCookie path].
		aCookie expiry isNil ifFalse: [
			stream
				nextPutAll: '; expires=';
				nextPutAll: aCookie expiryString]]! !

!ILSite methodsFor: 'private-serving' stamp: 'np 4/5/2010 17:55'!
decode: aString
	^self codec decode: aString! !

!ILSite methodsFor: 'defaults' stamp: 'sa 4/12/2010 14:48'!
defaultCodec
	^GRNullCodec new! !

!ILSite methodsFor: 'private-serving' stamp: 'np 4/5/2010 17:55'!
encode: aString
	^self codec encode: aString! !

!ILSite methodsFor: 'accessing' stamp: 'np 10/24/2008 14:19'!
helpResolve: aResolution 
	(self canAnswer and: [aResolution siteMatch: self]) ifFalse: [^nil].
	^self answerTo: aResolution request! !

!ILSite methodsFor: 'private-serving' stamp: 'np 5/1/2009 14:44'!
isBinaryResponse: aResponse
	| tokens type subTypes |
	tokens := aResponse contentType findTokens: '/'.
	type := tokens first.
	type = 'text' ifTrue: [^false].
	tokens size = 1 ifTrue: [^true].
	subTypes := tokens second findTokens: '+'.
	^subTypes noneSatisfy: [:each | 
		#('x-javascript' 'xml' 'json') includes: each]! !

!ILSite methodsFor: 'private-serving' stamp: 'np 12/4/2008 17:16'!
parseCookies: aSwazooRequest
	| cookies swazooCookies cookieLine |
	cookies := Dictionary new.
	cookieLine := aSwazooRequest cookie.
	cookieLine isEmptyOrNil ifTrue: [^cookies]. 
	swazooCookies := cookieLine findTokens: $;.
	swazooCookies do: [:each |
		| pair key value |
		pair := each findTokens: $=.
		key := SpStringUtilities trimBlanksFrom: pair first.
		value := pair size > 1
			ifTrue: [SpStringUtilities trimBlanksFrom:  pair second]
			ifFalse: [nil].
		cookies at: key put: value].
	^cookies! !

!ILSite methodsFor: 'private-serving' stamp: 'np 4/5/2010 18:21'!
parseFields: aSwazooRequest
	| fields |
	fields := Dictionary new.
	aSwazooRequest isPost
		ifTrue: [
			"break encapsulation"
			aSwazooRequest ensureFullRead.
			aSwazooRequest postData keysAndValuesDo: [:key :value |
				| previousData data |
				data := self convertPostDataFrom: value.
				previousData := fields at: key ifAbsent: [nil].
				previousData isNil
					ifTrue: [fields at: key put: data]
					ifFalse: [
						(previousData isKindOf: OrderedCollection)
							ifTrue: [previousData add: data]
							ifFalse: [fields at: key put: (OrderedCollection with: previousData with: data)]]]]
		ifFalse: [
			aSwazooRequest queries do: [ :each |
				fields at: each key put: each value]].
	^fields! !

Object subclass: #SwazooIliad
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Iliad-Swazoo'!

!SwazooIliad class methodsFor: 'start/stop' stamp: 'np 12/3/2008 15:26'!
startOn: aPort
	ILSite default port: aPort.
	SwazooServer startSite: ILSite defaultName! !

!SwazooIliad class methodsFor: 'start/stop' stamp: 'np 12/3/2008 15:26'!
stop
	SwazooServer stopSite: ILSite defaultName! !
